//*-------------------------------------------------------------------
//* Licensed Materials - Property of IBM
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2002
//*
//* US Government Users Restricted Rights - Use, duplication or
//* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//*
//*-------------------------------------------------------------------
//*
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its reliability,
//* serviceability or functionality.
//*
//*-------------------------------------------------------------------
//*
//* This class is will unpack a driver jar onto an installed instance of
//* WebSphere Commerce Business Edition.  The class must be located
//* in the jar that it is intended to unpack.  The MANIFEST.MF file must
//* specify this class as the Main-Class.  In order to run this class you
//* should issue "java -jar driver.jar" where driver.jar is the name of
//* jar file.
//*
//*-------------------------------------------------------------------

package com.mycompany.install;

import java.io.File;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/**
 * This class installs the sample driver onto an installed version of
 * WebSphere Commerce Business Edition.
 */
public final class InstallDriver
 {
  /**
   * Classloader for accessing install utilities.
   */
  public static URLClassLoader _utilitiesClassLoader = null;

  /**
   * Array of arguments passed to this class.
   */
  public String[] _args = null;

  /**
   * Target installation. Either "WAS" or "WSAD".
   */
  public String _target = null;

  /**
   * Reference to the jar file that contains this class.
   */
  public ZipFile _zipFile = null;

  /**
   * Main entry point for this class. This method will accept a target parameter of either "WAS" or "WSAD".
   * If WAS is specified then the sample will be installed into the WAS environment.
   * If WSAD is specified then the sample will be installed into the WSAD environment.
   *
   * @param args An array of arguments.
   */
  public static void main(String[] args)
   {
    // If installing on Windows, check to see if we should be installing onto WSAD or WAS.
    String target = null;
    if (InstallHelper.isWindows())
     {
      if (args != null && args.length > 0)
       {
        target = args[0];
       }
      while (true)
       {
        if (InstallHelper.INSTALL_DESTINATION_WAS.equalsIgnoreCase(target) ||
            InstallHelper.INSTALL_DESTINATION_WSAD.equalsIgnoreCase(target))
         {
          break;
         }
        System.out.println("Enter the target environment. Either WAS or WSAD.");
        target = InstallHelper.readLine().trim();
       }
     }
    else
     {
      target = InstallHelper.INSTALL_DESTINATION_WAS;
     }

    // Initialize the install helper utilities
    InstallHelper.init(target);

    // Construct an instance of the installer
    InstallDriver installDriver = new InstallDriver(args, target);

    // Execute the install
    installDriver.install();

    // Successful exit
    System.exit(0);
   }

  /**
   * Constructor for this class.
   *
   * @param args An array of command line arguments.
   * @param target Target environment. WAS or WSAD.
   */
  public InstallDriver(String[] args, String target)
   {
    _args = args;
    _target = target;

    // get the name of the jar file
    String jarFileName = null;
    try
     {
      jarFileName = getClass().getProtectionDomain().getCodeSource().getLocation().getFile();
     }
    catch (Exception e)
     {
     }
    while (jarFileName == null || jarFileName.trim().length() == 0)
     {
      System.out.println("Could not determine the name of the jar.");
      System.out.println("Enter the jar name.");
      jarFileName = InstallHelper.readLine();
     }

    // construct the ZipFile object
    try
     {
      _zipFile = new ZipFile(jarFileName.trim());
     }
    catch (Exception e)
     {
      System.out.println("Error reading jar file \"" + jarFileName + "\"");
     }
   }

  /**
   * Install the documents into their correct locations.
   * Entries that start with "ear" are installed in the installed enterprise application archive directory.
   * Entries that start with "wcs" are installed in the WebSphere Commerce Server root directory.
   */
  public void installEntries()
   {
    System.out.println("Installing files");
    Enumeration entries = _zipFile.entries();
    while (entries.hasMoreElements())
     {
      ZipEntry entry = (ZipEntry) entries.nextElement();
      if (!entry.isDirectory())
       {
        String name = entry.getName();
        if (name.startsWith("ear"))
         {
          for (int i = 0; i < InstallHelper._instances.size(); i++)
           {
            InstallHelper.InstanceInfo instanceInfo = (InstallHelper.InstanceInfo) InstallHelper._instances.elementAt(i);
            InstallHelper.installEntry(_zipFile, entry, instanceInfo._ear + name.substring("ear".length()));
           }
         }
        else if (name.startsWith("wcs"))
         {
          InstallHelper.installEntry(_zipFile, entry, InstallHelper._wcs + name.substring("wcs".length()));
         }
       }
     }
   }

  /**
   * Install the documents into their correct locations.
   * Entries that start with "workspace" are installed in the WSAD workspace directory.
   * Entries that start with "wcs" are installed in the WebSphere Commerce Development directory.
   */
  public void installWSADEntries()
   {
    System.out.println("Installing files");
    Enumeration entries = _zipFile.entries();
    while (entries.hasMoreElements())
     {
      ZipEntry entry = (ZipEntry) entries.nextElement();
      if (!entry.isDirectory())
       {
        String name = entry.getName();
        if (name.startsWith("workspace"))
         {
          for (int i = 0; i < InstallHelper._instances.size(); i++)
           {
            InstallHelper.InstanceInfo instanceInfo = (InstallHelper.InstanceInfo) InstallHelper._instances.elementAt(i);
            InstallHelper.installEntry(_zipFile, entry, instanceInfo._workspace + name.substring("workspace".length()));
           }
         }
        else if (name.startsWith("wcs"))
         {
          InstallHelper.installEntry(_zipFile, entry, InstallHelper._wcs + name.substring("wcs".length()));
         }
       }
     }
   }


  /**
   * Create a classloader for accessing xml4j.jar and installutil.jar.
   */
  public static void initUtilitiesClassLoader()
   {
    if (_utilitiesClassLoader == null)
     {
      // Add xml4j.jar to the classpath
      File xml4jFile = new File(InstallHelper._wcs + InstallHelper._fileSeparator + "lib" + InstallHelper._fileSeparator + "xml4j.jar");
      if (!xml4jFile.exists())
       {
        System.out.println("xml4j.jar could not be found as " + xml4jFile.getAbsolutePath());
        System.exit(-1);
       }
      // Add installutil.jar to the classpath
      File installutilFile = new File(InstallHelper._wcs + InstallHelper._fileSeparator + "lib" + InstallHelper._fileSeparator + "installutil.jar");
      if (!installutilFile.exists())
       {
        System.out.println("installutil.jar could not be found as " + installutilFile.getAbsolutePath());
        System.exit(-1);
       }
      try
       {
        URL xml4jURL = new URL("file", null, 0, xml4jFile.getAbsolutePath());
        URL installutilURL = new URL("file", null, 0, installutilFile.getAbsolutePath());
        URL urls[] = new URL[2];
        urls[0] = xml4jURL;
        urls[1] = installutilURL;
        _utilitiesClassLoader = new URLClassLoader(urls);
       }
      catch (Exception e)
       {
        e.printStackTrace();
        System.exit(-1);
       }
     }
   }

  /**
   * Update the commerce database.  This is done by invoking a bat file called
   * profileSamplePopulate.db2.bat in the WebSphere Commerce bin directory.
   *
   * @param instanceInfo Information about the instance being updated.
   */
  public void updateDataBase(InstallHelper.InstanceInfo instanceInfo)
   {
    System.out.println("Updating database");
    // Set up command as the script with appropriate parameters.
    // The script will update the database.
    String command;
    if (InstallHelper.isOS400())
     {
      command = InstallHelper._wcs + "/bin/profileSamplePopulatedb.os400.sh " +
                instanceInfo._databaseName + " " +
                instanceInfo._databaseUserId + " " +
                instanceInfo._databasePassword + " " +
                "/qibm/userdata/webcommerce/instances/" + instanceInfo._name;
     }
    else
     {
      if (!InstallHelper.isWindows())
       {
        if (instanceInfo._databaseType.equals(InstallHelper.DATABASE_TYPE_DB2))
         {
          command = InstallHelper._wcs + "/bin/profileSamplePopulatedb.db2.sh ";
         }
        else
         {
          command = InstallHelper._wcs + "/bin/profileSamplePopulatedb.oracle.sh ";
         }
       }
      else // Windows
       {
        if (instanceInfo._databaseType.equals(InstallHelper.DATABASE_TYPE_DB2))
         {
          command = InstallHelper._wcs + "\\bin\\profileSamplePopulatedb.db2.bat ";
         }
        else
         {
          command = InstallHelper._wcs + "\\bin\\profileSamplePopulatedb.oracle.bat ";
         }
       }
      command += instanceInfo._databaseName + " " +
                 instanceInfo._databaseUserId + " " +
                 instanceInfo._databasePassword + " " +
                 instanceInfo._databaseUserId.toUpperCase() + " " +
                 instanceInfo._errorLogDir + " " +
                 instanceInfo._wcaLogger;
     }
    // Launch the command.
    InstallHelper.executeCommand(command, true, instanceInfo._workingDirectory);
   }

  /**
   * Alter the database.
   *
   * @param instanceInfo Information about the instance being updated.
   */
  public void alterDataBase(InstallHelper.InstanceInfo instanceInfo)
   {
    System.out.println("Altering database");
    // Set up command with appropriate parameters.
    String command;
    if (InstallHelper.isOS400())
     {
      command = "db2 -v -S -t -r " + instanceInfo._databaseName +
                " -u " + instanceInfo._databaseUserId +
                " -p " + instanceInfo._databasePassword +
                " -f /qibm/proddata/webcommerce/schema/db2/profileSample.schema.sql " +
                instanceInfo._databaseUserId;
     }
    else
     {
      if (!InstallHelper.isWindows())
       {
        if (instanceInfo._databaseType.equals(InstallHelper.DATABASE_TYPE_DB2))
         {
          command = InstallHelper._wcs + "/bin/profileSampleCreatedb.db2.sh ";
         }
        else
         {
          command = InstallHelper._wcs + "/bin/profileSampleCreatedb.oracle.sh ";
         }
       }
      else // Windows
       {
        if (instanceInfo._databaseType.equals(InstallHelper.DATABASE_TYPE_DB2))
         {
          command = "db2cmd /c " + InstallHelper._wcs + "\\bin\\profileSampleCreatedb.db2.bat ";
         }
        else
         {
          command = InstallHelper._wcs + "\\bin\\profileSampleCreatedb.oracle.bat ";
         }
       }
      command += instanceInfo._databaseName + " " +
                 instanceInfo._databaseUserId + " " +
                 instanceInfo._databasePassword + " ";
      if (instanceInfo._databaseType.equals(InstallHelper.DATABASE_TYPE_DB2))
       {
        command += instanceInfo._databaseUserId.toUpperCase();
       }
     }
    // Launch the command.
    InstallHelper.executeCommand(command, true, instanceInfo._workingDirectory);
   }

  /**
   * Update the instance configuration file.
   *
   * @param instanceInfo Information about the instance being updated.
   */
  public void updateConfiguration(InstallHelper.InstanceInfo instanceInfo)
   {
    System.out.println("Updating instance configuration file.");
    InstallHelper.updateFile(instanceInfo._configXML,
                             "<resourceConfig file=\"segmentation/resources.xml\" />",
                             "<resourceConfig file=\"segmentation/MyResources.xml\" />");
   }

  /**
   * Update the segmentation properties file.
   *
   * @param instanceInfo Information about the instance being updated.
   */
  public void updateProperties(InstallHelper.InstanceInfo instanceInfo)
   {
    System.out.println("Updating properties file.");
    InstallHelper.updateFile(instanceInfo._ear + "/properties/com/ibm/commerce/tools/segmentation/Resources.properties",
                             "segmentNotebookAccountAmountSpentPanel=Sales",
                             "segmentNotebookAccountAmountSpentPanel=Sales" + InstallHelper._newLine + "segmentNotebookBeveragePanel=Preferred beverage");
   }

  /**
   * Get EJB module definition.
   *
   * @param instanceInfo Information about the instance being updated.
   * @param ejbJar Name of ejb jar. The EJB jar must be in the installed ear directory.
   */
  public InstallHelper.EJBModuleDefinition getEJBModuleDefinition(InstallHelper.InstanceInfo instanceInfo, String ejbJar)
   {
    initUtilitiesClassLoader();
    // need to load the utility class using a new class loader so it can find xml4j.jar.
    try
     {
      Class installEJBHelperClass = _utilitiesClassLoader.loadClass("com.mycompany.install.util.InstallEJBHelper");
      Class paramTypes[] = new Class[2];
      paramTypes[0] = instanceInfo.getClass();
      paramTypes[1] = Class.forName("java.lang.String");
      Method getEJBDefinitionsMethod = installEJBHelperClass.getMethod("getEJBModuleDefinition", paramTypes);
      Object paramValues[] = new Object[2];
      paramValues[0] = instanceInfo;
      paramValues[1] = ejbJar;
      return (InstallHelper.EJBModuleDefinition) getEJBDefinitionsMethod.invoke(null, paramValues);
     }
    catch (Exception e)
     {
      e.printStackTrace();
      System.out.println("Error getting definition for " + ejbJar);
      System.exit(0);
     }
    return null;
   }

  /**
   * Deploy the EJB.
   *
   * @param instanceInfo Information about the instance being updated.
   */
  public void deployEJB(InstallHelper.InstanceInfo instanceInfo)
   {
    System.out.println("Deploying EJB.");

    // Ensure that the Commerce server is stopped
    if (!InstallHelper.isOS400())
     {
      String command = InstallHelper._xmlconfig +
                       " -import " + InstallHelper._wcs + InstallHelper._fileSeparator + "bin" + InstallHelper._fileSeparator + "stopappserver.xml" +
                       " -adminNodeName " + instanceInfo._nodeName +
                       " -substitute \"DRIVER_NODE_NAME=" + InstallHelper._nameServiceHost + ";DRIVER_INSTANCE_NAME=" + instanceInfo._name +  "\"" +
                       " -nameServiceHost " + InstallHelper._nameServiceHost +
                       " -nameServicePort " + instanceInfo._wasPort;
      InstallHelper.executeCommand(command, true, instanceInfo._workingDirectory);
     }

    // Generate deployed code for the EJB
    String command = InstallHelper._ejbdeploy + " " +
                     instanceInfo._ear + InstallHelper._fileSeparator + "beverageEJB." + instanceInfo._databaseType + ".jar " +
                     instanceInfo._workingDirectory + " " +
                     instanceInfo._ear + InstallHelper._fileSeparator + "beverageEJB-deployed.jar";
    InstallHelper.executeCommand(command, true, instanceInfo._workingDirectory);
    InstallHelper.rename(instanceInfo._ear + InstallHelper._fileSeparator + "beverageEJB-deployed.jar", instanceInfo._ear + InstallHelper._fileSeparator + "beverageEJB.jar", true);

    // Add the ejb to the application
    Vector ejbs = new Vector();
    ejbs.addElement(getEJBModuleDefinition(instanceInfo, "beverageEJB.jar"));
    InstallHelper.addEJBsToEAR(instanceInfo, ejbs);
   }

  /**
   * Install the sample.
   */
  public void install()
   {
    // Based on target environment, call appropriate install method.
    if (_target.equalsIgnoreCase(InstallHelper.INSTALL_DESTINATION_WAS))
     {
      installWAS();
     }
    else if (_target.equalsIgnoreCase(InstallHelper.INSTALL_DESTINATION_WSAD))
     {
      installWSAD();
     }
   }

  /**
   * Install the driver into a WAS environment.
   */
  public void installWAS()
   {
    // lay down the file assets
    installEntries();

    // loop through the instances
    for (int i = 0; i < InstallHelper._instances.size(); i++)
     {
      InstallHelper.InstanceInfo instanceInfo = (InstallHelper.InstanceInfo) InstallHelper._instances.elementAt(i);

      // add table to the database
      alterDataBase(instanceInfo);

      // update bootstrap information in database
      updateDataBase(instanceInfo);

      // update instance configuration file
      updateConfiguration(instanceInfo);

      // update properties file
      updateProperties(instanceInfo);

      // deploy the EJB
      deployEJB(instanceInfo);

      // patch web.xml files
      InstallHelper.patchWebModules(instanceInfo);
     }
    System.out.println("Installation complete");
   }

  /**
   * Install the driver into a WSAD environment.
   */
  public void installWSAD()
   {
    // lay down file assets
    installWSADEntries();

    // loop through instances
    for (int i = 0; i < InstallHelper._instances.size(); i++)
     {
      InstallHelper.InstanceInfo instanceInfo = (InstallHelper.InstanceInfo) InstallHelper._instances.elementAt(i);

      // import the new projects into the workspace
      InstallHelper.importProjects(instanceInfo);

      // Add a java project to the workspace cofiguration.
      InstallHelper.addWSADJavaProject(instanceInfo, "wcs.examples", "lib/wcsexamples.jar");

      // Add an EJB project to the workspace configuration.
      InstallHelper.addWSADEJBProject(instanceInfo, "WCSExample-ejb", "WCSExample-ejb.jar");

      // Update the instance configuration.
      updateConfiguration(instanceInfo);

      // Add new table to the database.
      alterDataBase(instanceInfo);

      // Add bootstrap information to the database.
      updateDataBase(instanceInfo);
     }
    System.out.println("Installation into WSAD complete");
   }

 }
