//*-------------------------------------------------------------------
//* Licensed Materials - Property of IBM
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2002
//*
//* US Government Users Restricted Rights - Use, duplication or
//* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//*
//*-------------------------------------------------------------------
//*
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its reliability,
//* serviceability or functionality.
//*
//*-------------------------------------------------------------------
//*
//* This class is contains helper methods that are used during the install.
//*
//*-------------------------------------------------------------------

package com.mycompany.install;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.util.Date;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/**
 * This class contains a number of utility methods that are used during the install.
 * This class must not be dependant on anything that is not packaged in the installer jar (or part of the JDK).
 * If you reference classes outside the jar, then the self installing jar will not be able to see them.
 */
public final class InstallHelper
 {
  /**
   * Install Destination is WAS.
   */
  public final static String INSTALL_DESTINATION_WAS = "WAS";

  /**
   * Install Destination is WSAD.
   */
  public final static String INSTALL_DESTINATION_WSAD = "WSAD";

  /**
   * DB2 database type.
   */
  public final static String DATABASE_TYPE_DB2 = "DB2";

  /**
   * Oracle database type.
   */
  public final static String DATABASE_TYPE_ORACLE = "Oracle";

  /**
   * Location of WebSphere Commerce Server root directory.
   */
  public static String _wcs = null;

  /**
   * Location of WebSphere Application Server root directory.
   */
  public static String _was = null;

  /**
   * Name service host.
   */
  public static String _nameServiceHost = null;

  /**
   * XMLConfig executable.
   */
  public static String _xmlconfig = null;

  /**
   * ejbdeploy executable.
   */
  public static String _ejbdeploy = null;

  /**
   * Name of the WebSphere Commerce instances that should be configured by this install.
   */
  public static Vector _instances = new Vector();

  /**
   * Operating system.
   */
  public static String _os = System.getProperty("os.name");

  /**
   * Line separator character sequence for this platform.
   */
  public static String _newLine = System.getProperty("line.separator");

  /**
   * File separator.
   */
  public static String _fileSeparator = System.getProperty("file.separator");

  /**
   * Character buffer used to modify text files.
   */
  public static char[] _charBuffer = new char[10000];

  /**
   * Byte buffer used to unzip jar entries.
   */
  public static byte[] _buffer = new byte[10000];

  /**
   * Standard input stream reader.
   */
  public static InputStreamReader _in = new InputStreamReader(System.in);

  /**
   * Value used to generate unique identifiers.
   */
  private static long _lastTime = 0;

  /**
   * Initialize for target environment.
   */
  public static void init(String target)
   {
    init(target, false);
   }

  /**
   * Initialize for target environment.
   */
  public static void init(String target, boolean noDatabase)
   {
    // Based on the target specified, call the appropriate initialization methods.
    if (INSTALL_DESTINATION_WSAD.equals(target))
     {
      initWCSDev(null);
      initWSADInstances(noDatabase);
     }
    else
     {
      initWCS(null);
      initWAS();
      initInstances(noDatabase);
     }
   }

  /**
   * Locates and validates the root directory for WebSphere Commerce.
   *
   * @param wcs Location of root directory for WebSphere Commerce. If this is null, then
   *            this location will be determined algorithmically.
   */
  public static void initWCS(String wcs)
   {
    if (isWindows())
     {
      // On Windows, the location of WebSphere Commerce may not be where we expect it.
      // Use the passed in value if available, or ask the user for the location.
      if (_wcs == null)
       {
        if (wcs == null)
         {
          _wcs = "C:\\Program Files\\WebSphere\\CommerceServer";
         }
        else
         {
          _wcs = wcs;
         }
       }
      while (true)
       {
        if (_wcs != null && _wcs.length() > 0)
         {
          File instancesRoot = new File(_wcs + "\\instances");
          if (instancesRoot.exists() && instancesRoot.isDirectory())
           {
            break;
           }
         }
        System.out.println("The root directory for WebSphere Commerce Server was not found at: "+_wcs);
        System.out.println("Specify the root directory for WebSphere Commerce Server.");
        _wcs = readLine().trim();
       }
     }
    else if (isAIX())
     {
      // We know where it should be on AIX
      _wcs = "/usr/WebSphere/CommerceServer";
     }
    else if (isSun() || isLinux())
     {
      // We know where it should be on Sun and Linux
      _wcs = "/opt/WebSphere/CommerceServer";
     }
    else if (isOS400())
     {
      // We know where it should be on I-Series
      _wcs = "/qibm/proddata/webcommerce";
     }
    else
     {
      System.out.println("Operating System could not be determined.");
      System.exit(-1);
     }
    // Perform some simple validation that this is a valid commerce root by trying
    // to detect the instances directory.
    File instancesRoot = new File(_wcs + _fileSeparator + "instances");
    if (isOS400())
     {
      instancesRoot = new File ("/qibm/userdata/webcommerce/instances");
     }
    if (!instancesRoot.exists() || !instancesRoot.isDirectory())
     {
      System.out.println("Could not find root directory of WebSphere Commerce Server 5.4.");
      System.exit(-1);
     }
   }

  /**
   * Locates and validates the root directory for WebSphere Commerce when installing into the
   * development environment.
   *
   * @param wcs Location of root directory for WebSphere Commerce. If this is null, then
   *            this location will be determined algorithmically.
   */
  public static void initWCSDev(String wcs)
   {
    if (isWindows())
     {
      // If the location of WebSphere Commerce may not be where we expect it,
      // use the passed in value if available, or ask the user for the location.
      if (_wcs == null)
       {
        if (wcs == null)
         {
          _wcs = "C:\\CommerceStudioEnhancements";
         }
        else
         {
          _wcs = wcs;
         }
       }
      while (true)
       {
        if (_wcs != null && _wcs.length() > 0)
         {
          File instancesRoot = new File(_wcs + "\\instances");
          if (instancesRoot.exists() && instancesRoot.isDirectory())
           {
            break;
           }
         }
        System.out.println("The WebSphere Commerce Studio Enhancements directory was not found at: "+_wcs);
        System.out.println("Specify WebSphere Commerce Studio Enhancements directory.");
        _wcs = readLine().trim();
       }
     }
    else
     {
      System.out.println("Operating System must be windows.");
      System.exit(-1);
     }
    // Perform a simple check to ensure that the directory is valid. Look for the instances directory.
    File instancesRoot = new File(_wcs + _fileSeparator + "instances");
    if (!instancesRoot.exists() || !instancesRoot.isDirectory())
     {
      System.out.println("Could not find root directory of WebSphere Commerce Server 5.4.");
      System.exit(-1);
     }
   }

  /**
   * Locates and validate the WebSphere Application Server root directory.
   */
  public static void initWAS()
   {
    if (isWindows())
     {
      // On Windows parse the setenv.bat file to find out where the WAS root is.
      String setenvFile = _wcs + "\\bin\\" + "setenv.bat";
      try
       {
        FileReader fileReader = new FileReader(setenvFile);
        BufferedReader bufferedReader = new BufferedReader(fileReader);
        String line = bufferedReader.readLine();
        while (line != null)
         {
          if (line.startsWith("SET WAS_HOME="))
           {
            _was = line.substring("SET WAS_HOME=".length());
            break;
           }
          line = bufferedReader.readLine();
         }
        bufferedReader.close();
        fileReader.close();
       }
      catch (Exception e)
       {
        System.out.println("Error accessing \"" + setenvFile + "\"");
        System.exit(-1);
       }
      if (_was == null)
       {
        System.out.println("Unable to locate the WebSphere Application root directory.");
        System.exit(-1);
       }

      // Set the path to the XMLConfig executable
      _xmlconfig = _was + "\\bin\\XMLConfig.bat";

      // Set the path to the ejbdeploy executable
      _ejbdeploy = _was + "\\bin\\ejbdeploy.bat";
     }
    else if (isAIX())
     {
      // On AIX we know where things are.
      _was = "/usr/WebSphere/AppServer";
      _xmlconfig = _was + "/bin/XMLConfig.sh";
      _ejbdeploy = _was + "/bin/ejbdeploy.sh";
     }
    else if (isSun() || isLinux())
     {
      // On Sun and Linux we know where things are.
      _was = "/opt/WebSphere/AppServer";
      _xmlconfig = _was + "/bin/XMLConfig.sh";
      _ejbdeploy = _was + "/bin/ejbdeploy.sh";
     }
    else if (isOS400())
     {
      // On OS400 we know where the executables are.
      _xmlconfig = "/Qibm/ProdData/WebASAdv4/bin/XMLConfig";
      _ejbdeploy = "/qibm/proddata/webasadv4/bin/ejbdeploy";
     }
    else
     {
      System.out.println("Operating System could not be determined.");
      System.exit(-1);
     }

    if (!isOS400())
     {
      // if not I-Series, do a simple check to ensure that the installedApps directory exists.
      File installedAppsRoot = new File(_was + _fileSeparator + "installedApps");
      if (!installedAppsRoot.exists() || !installedAppsRoot.isDirectory())
       {
        System.out.println("Could not find root directory of WebSphere Application Server");
        System.exit(-1);
       }

     }
    // Get the name service host used when calling XMLConfig
    try
     {
      _nameServiceHost = InetAddress.getLocalHost().getHostName();
      int index;
      if ((index = _nameServiceHost.indexOf(".")) != -1)
       {
        _nameServiceHost = _nameServiceHost.substring(0, index);
       }
     }
    catch (Exception e)
     {
      e.printStackTrace();
      System.exit(-1);
     }
   }

  /**
   * Locates the list of instances.
   *
   * @param noDatabase Set this flag to true if you do not intend to access the database. This will skip
   *                   the logic that attempts to determine the database password.
   */
  public static void initInstances(boolean noDatabase)
   {
    // Based on operating system, delegate to the appropriate method
    if (!isOS400())
     {
      initInstancesOnWINUnix(noDatabase);
     }
    else
     {
      initInstanceOnOS400(noDatabase);
     }
   }

  /**
   * Locates the list of instances on Windows and Unix platform.
   *
   * @param noDatabase Set this flag to true if you do not intend to access the database. This will skip
   *                   the logic that attempts to determine the database password.
   */
  public static void initInstancesOnWINUnix(boolean noDatabase)
   {
    // Parse the wcs_instances file to get a list of instances.
    String wcs_instances = _wcs + _fileSeparator + "instances" + _fileSeparator + "wcs_instances";
    Vector instanceNames = new Vector();
    try
     {
      FileReader fileReader = new FileReader(wcs_instances);
      BufferedReader bufferedReader = new BufferedReader(fileReader);
      String line = bufferedReader.readLine();
      while (line != null)
       {
        if (line.startsWith("[instance]"))
         {
          line = bufferedReader.readLine();
          while (line != null)
           {
            if (line.trim().length() > 0)
             {
              instanceNames.addElement(line.substring(0, line.indexOf(";")));
             }
            line = bufferedReader.readLine();
           }
          break;
         }
        line = bufferedReader.readLine();
       }
      bufferedReader.close();
      fileReader.close();
     }
    catch (Exception e)
     {
      System.out.println("Error accessing \"" + wcs_instances + "\"");
      System.exit(-1);
     }

    // If there are no instances, then quit.
    if (instanceNames.size() == 0)
     {
      System.out.println("No instances were found. Use Configuration Manager to create an instance.");
      System.exit(-1);
     }

    // Loop through and initialize the instances.
    for (int i = 0; i < instanceNames.size(); i++)
     {
      // construct an InstanceInfo object for storing information about the instance
      InstanceInfo instanceInfo = new InstanceInfo();

      // add the instance to a Vector of instances.
      _instances.addElement(instanceInfo);

      // set the name of the instance
      instanceInfo._name = (String) instanceNames.elementAt(i);

      // set the path to the <instance>.xml file
      instanceInfo._configXML = _wcs + _fileSeparator + "instances" + _fileSeparator + instanceInfo._name + _fileSeparator + "xml" + _fileSeparator + instanceInfo._name + ".xml";

      // ensure that the <instance>.xml file exists
      File instanceConfigFile = new File(instanceInfo._configXML);
      if (!instanceConfigFile.exists())
       {
        System.out.println("Could not locate instance configuration file \"" + instanceInfo._configXML + "\".");
        System.exit(-1);
       }

      // set the path to the installed ear directory
      instanceInfo._ear = _was + "/installedApps/WC_Enterprise_App_" + instanceInfo._name + ".ear";
      while (true)
       {
        if (instanceInfo._ear != null && instanceInfo._ear.length() > 0)
         {
          File earFile = new File(instanceInfo._ear);
          if (earFile.exists() && earFile.isDirectory())
           {
            break;
           }
         }
        // If the instance directory does not exist, ask for it.
        System.out.println("The WebSphere Commerce Enterprise Application directory was not found at: " + instanceInfo._ear);
        System.out.println("Specify WebSphere Commerce Enterprise Application directory for instance " + instanceInfo._name + ".");
        instanceInfo._ear = readLine().trim();
       }
      // make sure the ear can be renamed. The rename method will exit if it fails.
      rename(instanceInfo._ear, instanceInfo._ear + ".bak", false);
      rename(instanceInfo._ear + ".bak", instanceInfo._ear, false);

      // set the logger configuaration file for massloader
      instanceInfo._wcaLogger = _wcs + "/instances/" + instanceInfo._name + "/xml/loader/WCALoggerConfig.xml";

      // set the error log directory for massloader
      instanceInfo._errorLogDir = _wcs + "/instances/" + instanceInfo._name + "/logs";

      // Parse <instance>.xml file for information
      StringBuffer content = getFileContent(instanceInfo._configXML);
      // Get database user id
      instanceInfo._databaseUserId = getAttribute(content, "DBUserID");
      if (instanceInfo._databaseUserId == null)
       {
        System.out.println("Could not determine database userId for instance " + instanceInfo._name);
        System.exit(-1);
       }
      // get database type
      instanceInfo._databaseType = getAttribute(content, "DBMSName");
      if (instanceInfo._databaseType == null)
       {
        System.out.println("Could not determine database type for instance " + instanceInfo._name);
        System.exit(-1);
       }
      // get database name
      if (instanceInfo._databaseType.equals(DATABASE_TYPE_DB2))
       {
        instanceInfo._databaseName = getAttribute(content, "name", "<DB");
       }
      else
       {
        instanceInfo._databaseName = getAttribute(content, "ServiceName");
       }
      if (instanceInfo._databaseName == null)
       {
        System.out.println("Could not determine database name for instance " + instanceInfo._name);
        System.exit(-1);
       }
      // get the database password
      while (true)
       {
        if (!noDatabase && (instanceInfo._databasePassword == null || instanceInfo._databasePassword.length() == 0))
         {
          System.out.println("Specify the password that should be used with user \"" + instanceInfo._databaseUserId + "\".");
          instanceInfo._databasePassword = readLine().trim();
         }
        else
         {
          break;
         }
       }
      // get the port used to access WAS. This will be used when calling XMLConfig.
      instanceInfo._wasPort = getAttribute(content, "port");
      content = getFileContent(_wcs + _fileSeparator + "instances" + _fileSeparator + instanceInfo._name + _fileSeparator + "xml" + _fileSeparator + instanceInfo._name + "_was.xml");
      // get the node name.
      instanceInfo._nodeName = getAttribute(content, "<node action=\"update\" name");
      if (instanceInfo._nodeName == null)
       {
        System.out.println("Could not determine node name for instance " + instanceInfo._name);
        System.exit(-1);
       }
      // set up a working directory. This is used when deploying EJBs.
      instanceInfo._workingDirectory = _wcs + _fileSeparator + "instances" + _fileSeparator + instanceInfo._name + _fileSeparator + "work";
      new File(instanceInfo._workingDirectory).mkdir();
     }
   }

  /**
   * Locates the instance for OS400.
   *
   * @param noDatabase Set this flag to true if you do not intend to access the database. This will skip
   *                   the logic that attempts to determine the database password.
   */
  public static void initInstanceOnOS400(boolean noDatabase)
   {
    // Get the instance that should be updated.
    System.out.println("Specify the WCS instance name for this install.");
    String instanceName = readLine().trim();
    String configXML = null;

    // Verify that the instance name is valid.
    String wcs_instances = "/qibm/userdata/webcommerce/instances/wcs_instances";
    try
     {
      FileReader fileReader = new FileReader(wcs_instances);
      BufferedReader bufferedReader = new BufferedReader(fileReader);
      String line = bufferedReader.readLine();
      while (line != null)
       {
        if (line.startsWith(instanceName + ";"))
         {
          configXML = line.substring(line.indexOf("=")+1);
          break;
         }
        line = bufferedReader.readLine();
       }
      bufferedReader.close();
      fileReader.close();
     }
    catch (Exception e)
     {
      System.out.println("Error accessing \"" + wcs_instances + "\"");
      System.exit(-1);
     }
    if (configXML == null || configXML.length() == 0)
     {
      System.out.println(instanceName + " was not found.");
      System.exit(-1);
     }

    // construct InstanceInfo object and add it to the vector of instances.
    InstanceInfo instanceInfo = new InstanceInfo();
    _instances.addElement(instanceInfo);
    // set the name of the instance
    instanceInfo._name = instanceName;
    // set the path to the <instance>.xml file
    instanceInfo._configXML = configXML;
    // Ensure that the <instance>.xml file exists.
    File instanceConfigFile = new File(instanceInfo._configXML);
    if (!instanceConfigFile.exists())
     {
      System.out.println("Could not locate instance configuration file \"" + instanceInfo._configXML + "\".");
      System.exit(-1);
     }
    // set the path to the massload log configuration file.
    instanceInfo._wcaLogger = "/qibm/userdata/webcommerce/instances/" + instanceInfo._name + "/xml/loader/WCALoggerConfig.xml";
    // set the path to the error log directory
    instanceInfo._errorLogDir = "/qibm/userdata/webcommerce/instances/" + instanceInfo._name + "/logs";

    // Parse the <instance>.xml file for useful information
    StringBuffer content = getFileContent(instanceInfo._configXML);
    instanceInfo._os400WASInstanceName = getAttribute(content, "AdminServer");
    // Set the root director for the application server instance
    _was = "/qibm/userdata/webasadv4/" + instanceInfo._os400WASInstanceName;
    File installedAppsRoot = new File(_was + _fileSeparator + "installedApps");
    if (!installedAppsRoot.exists() || !installedAppsRoot.isDirectory())
     {
      System.out.println("Could not find root directory of WebSphere Application Server");
      System.exit(-1);
     }
    // Set the location of the installed ear directory for this instance.
    instanceInfo._ear = _was + "/installedApps/WC_Enterprise_App_" + instanceInfo._name + ".ear";

    // Set the database user id.
    instanceInfo._databaseUserId = getAttribute(content, "DBUserID");
    if (instanceInfo._databaseUserId == null)
     {
      System.out.println("Could not determine database userId for instance " + instanceInfo._name);
      System.exit(-1);
     }
    // set the database type.
    instanceInfo._databaseType = "OS400";
    // set the database name.
    instanceInfo._databaseName = getAttribute(content, "  name");
    if (instanceInfo._databaseName == null)
     {
      System.out.println("Could not determine database name for instance " + instanceInfo._name);
      System.exit(-1);
     }

    // Get the database password.
    while (true)
     {
      if (!noDatabase && (instanceInfo._databasePassword == null || instanceInfo._databasePassword.length() == 0))
       {
        System.out.println("Specify the password that should be used with user \"" + instanceInfo._databaseUserId + "\".");
        instanceInfo._databasePassword = readLine().trim();
       }
      else
       {
        break;
       }
     }

    // get the port used to access WAS. This will be used when calling XMLConfig.
    instanceInfo._wasPort = getAttribute(content, "port");
    content = getFileContent("/qibm/userdata/webcommerce/instances/" + instanceInfo._name + "/xml/" + instanceInfo._name + "_was.xml");
    // get the node name.
    instanceInfo._nodeName = getAttribute(content, "<node action=\"update\" name");
    if (instanceInfo._nodeName == null)
     {
      System.out.println("Could not determine node name for instance " + instanceInfo._name);
      System.exit(-1);
     }

    // set up a working directory.
    instanceInfo._workingDirectory = "/qibm/userdata/webcommerce/instances/" + instanceInfo._name + "/work";
    new File(instanceInfo._workingDirectory).mkdir();
   }

  /**
   * Locates the list of instances used in the development environment.
   *
   * @param noDatabase Set this flag to true if you do not intend to access the database. This will skip
   *                   the logic that attempts to determine the database password.
   */
  public static void initWSADInstances(boolean noDatabase)
   {
    // Parse the wcs_instances file to get a list of instances.
    String wcs_instances = _wcs + _fileSeparator + "instances" + _fileSeparator + "wcs_instances";
    Vector instanceNames = new Vector();
    try
     {
      FileReader fileReader = new FileReader(wcs_instances);
      BufferedReader bufferedReader = new BufferedReader(fileReader);
      String line = bufferedReader.readLine();
      while (line != null)
       {
        if (line.startsWith("[instance]"))
         {
          line = bufferedReader.readLine();
          while (line != null)
           {
            if (line.trim().length() > 0)
             {
              instanceNames.addElement(line.substring(0, line.indexOf(";")));
             }
            line = bufferedReader.readLine();
           }
          break;
         }
        line = bufferedReader.readLine();
       }
      bufferedReader.close();
      fileReader.close();
     }
    catch (Exception e)
     {
      System.out.println("Error accessing \"" + wcs_instances + "\"");
      System.exit(-1);
     }
    // If no instances were found, then quit.
    if (instanceNames.size() == 0)
     {
      System.out.println("No instances were found. Use createInstance.bat to create an instance.");
      System.exit(-1);
     }
    // loop through the instances.
    for (int i = 0; i < instanceNames.size(); i++)
     {
      // Construct InstanceInfo object and add it to the list of instances.
      InstanceInfo instanceInfo = new InstanceInfo();
      _instances.addElement(instanceInfo);
      // Set the instance name
      instanceInfo._name = (String) instanceNames.elementAt(i);
      // Set the path to the <instance>.xml file.
      instanceInfo._configXML = _wcs + _fileSeparator + "instances" + _fileSeparator + instanceInfo._name + _fileSeparator + "xml" + _fileSeparator + instanceInfo._name + ".xml";
      // Verify that <instance>.xml exists.
      File instanceConfigFile = new File(instanceInfo._configXML);
      if (!instanceConfigFile.exists())
       {
        System.out.println("Could not locate instance configuration file \"" + instanceInfo._configXML + "\".");
        System.exit(-1);
       }
      // Set the path to the massloader log config file.
      instanceInfo._wcaLogger = _wcs + "/instances/" + instanceInfo._name + "/xml/loader/WCALoggerConfig.xml";
      // Set the path to the log directory.
      instanceInfo._errorLogDir = _wcs + "/instances/" + instanceInfo._name + "/logs";

      // Parse <instance>.xml to get some useful information.
      StringBuffer content = getFileContent(instanceInfo._configXML);
      // Retrieve the database user id.
      instanceInfo._databaseUserId = getAttribute(content, "DBUserID");
      if (instanceInfo._databaseUserId == null)
       {
        System.out.println("Could not determine database userId for instance " + instanceInfo._name);
        System.exit(-1);
       }
      // Get the database type.
      instanceInfo._databaseType = getAttribute(content, "DBMSName");
      if (instanceInfo._databaseType == null)
       {
        System.out.println("Could not determine database type for instance " + instanceInfo._name);
        System.exit(-1);
       }
      // Get the database name.
      if (instanceInfo._databaseType.equals(DATABASE_TYPE_DB2))
       {
        instanceInfo._databaseName = getAttribute(content, "  name");
       }
      else
       {
        instanceInfo._databaseName = getAttribute(content, "ServiceName");
       }
      if (instanceInfo._databaseName == null)
       {
        System.out.println("Could not determine database name for instance " + instanceInfo._name);
        System.exit(-1);
       }
      // Get the database password.
      while (true)
       {
        if (!noDatabase && (instanceInfo._databasePassword == null || instanceInfo._databasePassword.length() == 0))
         {
          System.out.println("Specify the password that should be used with user \"" + instanceInfo._databaseUserId + "\".");
          instanceInfo._databasePassword = readLine().trim();
         }
        else
         {
          break;
         }
       }
      // Get the location of the workspace.
      instanceInfo._workspace = getAttribute(content, "StoresDocRoot");
      instanceInfo._workspace = instanceInfo._workspace.substring(0, instanceInfo._workspace.indexOf("\\wcstores"));
     }
   }

  /**
   * Reads a line of text from standard input.
   *
   * @return A line of text entered by the user.
   */
  public static String readLine()
   {
    String line = "";
    try
     {
      char c = (char) _in.read();
      while (c == '\n' || c == '\r')
       {
        c = (char) _in.read();
       }
      while (c != '\n' && c != '\r')
       {
        line += c;
        c = (char) _in.read();
       }
     }
    catch (Exception e2)
     {
     }
    return line;
   }

  /**
   * Return true if the operating system is Windows.
   */
  public static boolean isWindows()
   {
    return _os.toUpperCase().indexOf("WINDOWS") != -1;
   }

  /**
   * Return true if the operating system is AIX.
   */
  public static boolean isAIX()
   {
    return _os.toUpperCase().indexOf("AIX") != -1;
   }

  /**
   * Return true if the operating system is Sun
   */
  public static boolean isSun()
   {
    return _os.toUpperCase().indexOf("SUN") != -1;
   }

  /**
   * Return true if the operating system is Linux.
   */
  public static boolean isLinux()
   {
    return _os.toUpperCase().indexOf("LINUX") != -1;
   }

  /**
   * Return true if the operating system is OS400.
   */
  public static boolean isOS400()
   {
    return _os.toUpperCase().indexOf("OS/400") != -1;
   }

  /**
   * Determine if the specified file should be treated as a binary file.
   *
   * @param filename The name of the file to be tested.
   *
   * @return <code>true</code> if the file is a binary file.
   */
  public static boolean isBinary(String filename)
   {
    return filename.endsWith(".class") ||
           filename.endsWith(".jar") ||
           filename.endsWith(".ear") ||
           filename.endsWith(".zip") ||
           filename.endsWith(".war") ||
           filename.endsWith(".jpg") ||
           filename.endsWith(".gif");
   }

  /**
   * Rename a file. This method terminates the install if the file or directory can not be renamed.
   *
   * @param oldName The old name of the file.
   * @param oldName The new name for the file.
   * @param force Set to true if you want to try to delete newName if it exists.
   *
   */
  public static void rename(String oldName, String newName, boolean force)
   {
    File oldFile = new File(oldName);
    File newFile = new File(newName);
    if (newFile.exists())
     {
      if (force)
       {
        newFile.delete();
        if (newFile.exists())
         {
          System.out.println("Unable to delete " + newName + ".");
          System.exit(-1);
         }
       }
      else
       {
        System.out.println("Unable to rename " + oldName + " to " + newName + " because " + newName + " already exists.");
        System.exit(-1);
       }
     }
    try
     {
      oldFile.renameTo(newFile);
      if (!newFile.exists())
       {
        System.out.println(newFile + " could not be renamed to " + oldFile + ".");
        System.exit(-1);
       }
     }
    catch (Exception e)
     {
      System.out.println(newFile + " could not be renamed to " + oldFile + ".");
      System.exit(-1);
     }
   }

  /**
   * Executes a specified command.
   *
   * @param command The command to be executed.
   * @param log Whether the output and error msg from the command will be printed out.
   * @param tempDirectory Directory to be used to create temporary files.
   */
  public static int executeCommand(String command, boolean log, String tempDirectory)
   {
    if (log)
     {
      System.out.println(command);
     }
    try
     {
      Runtime runtime = Runtime.getRuntime();
      if (!isWindows() && command.indexOf('\"') != -1)
       {
        // Wrap commands that quote parameters in a shell script
        StringBuffer commandWrapper = new StringBuffer("#!/usr/bin/sh\n");
        commandWrapper.append(command);
        commandWrapper.append("\n");
        command = tempDirectory + _fileSeparator + "commandWrapper.sh";
        writeFileContent(tempDirectory + _fileSeparator + "commandWrapper.sh", commandWrapper);
        executeCommand("chmod 744 " + command);
       }
      Process p = runtime.exec(command);
      StdErrThread stdErrThread = new StdErrThread(p.getErrorStream(), log);
      stdErrThread.start();
      InputStream in = p.getInputStream();
      int c = in.read();
      while (c != -1)
       {
        if (log)
         {
          System.out.print((char)c);
         }
        c = in.read();
       }
      int result = p.waitFor();
      return result;
     }
    catch (Exception e)
     {
      System.out.println("Exception while executing "+command);
      e.printStackTrace();
     }
    return 0;
   }

  // Thread used for loggin standard err.
  public final static class StdErrThread extends Thread
   {
    InputStream _err;
    boolean _log;

    public StdErrThread(InputStream err, boolean log)
     {
      _err = err;
      _log = log;
     }

    public void run()
     {
      try
       {
        int c = _err.read();
        while (c != -1)
         {
          if (_log)
           {
            System.err.print((char)c);
           }
          c = _err.read();
         }
       }
      catch (Exception e)
       {
        e.printStackTrace();
       }
     }
   }

  /**
   * Executes a specified command.
   *
   * @param command The command to be executed.
   */
  public static void executeCommand(String command)
   {
    executeCommand(command, false, null);
   }

  /**
   * Strip "\r" characters from the specified buffer.
   *
   * @param buffer An array of bytes.
   * @param len Current length.
   *
   * @return New length.
   */
  public static int stripCarriageReturns(byte buffer[], int len)
   {
    for (int i = 0; i < len; i++)
     {
      if (buffer[i] == '\r')
       {
        System.arraycopy(buffer, i + 1, buffer, i, len - i - 1);
        len--;
       }
     }
    return len;
   }

  /**
   * Install a zip entry to a specified location.
   *
   * @param entry A ZipEntry object
   * @param targetName The target name of the file.
   */
  public static void installEntry(ZipFile zipFile, ZipEntry entry, String targetName)
   {
    try
     {
      InputStream inputStream = zipFile.getInputStream(entry);
      File targetFile = new File(targetName);
      targetFile.getParentFile().mkdirs();
      FileOutputStream outputStream = new FileOutputStream(targetName);
      int len = inputStream.read(_buffer);
      while (len != -1)
       {
        if (!isWindows() && !isBinary(targetName))
         {
          len = stripCarriageReturns(_buffer, len);
         }
        outputStream.write(_buffer, 0, len);
        len = inputStream.read(_buffer);
       }
      inputStream.close();
      outputStream.close();
      // ensure that shell scripts are executable
      if (!isWindows() && targetName.endsWith(".sh"))
       {
        executeCommand("chmod 744 " + targetName);
       }
     }
    catch (Exception e)
     {
      System.out.println("exception while extracting "+targetName);
      System.out.println(e);
     }
   }

  /**
   * Get the content of the specified file.
   *
   * @param fileName Name of file.
   *
   * @return Content of the file.
   */
  public static StringBuffer getFileContent(String fileName)
   {
    StringBuffer content = new StringBuffer();
    try
     {
      FileReader fileReader = new FileReader(fileName);
      int len = fileReader.read(_charBuffer);
      while (len > 0)
       {
        content.append(_charBuffer, 0, len);
        len = fileReader.read(_charBuffer);
       }
      fileReader.close();
     }
    catch (Exception e)
     {
      System.out.println(e);
      e.printStackTrace();
     }
    return content;
   }

  /**
   * Write out the content of a file.
   *
   * @param fileName Name of file.
   * @param content Content of the file.
   */
  public static void writeFileContent(String fileName, StringBuffer content)
   {
    try
     {
        FileWriter fileWriter = new FileWriter(fileName + ".new");
        fileWriter.write(content.toString());
        fileWriter.close();
        File oldFile = new File(fileName);
        oldFile.delete();
        File newFile = new File(fileName + ".new");
        newFile.renameTo(oldFile);
     }
    catch (Exception e)
     {
      e.printStackTrace();
      System.out.println("Unable to write out " + fileName);
      System.exit(-1);
     }
   }

  /**
   * Modify a string in a specified file.
   *
   * @param fileName The name of the file that is to be updated.
   * @param fromString The original string.
   * @param toString The replacement text.
   */
  public static void updateFile(String fileName, String fromString, String toString)
   {
    try
     {
      StringBuffer content = getFileContent(fileName);
      int toIndex = content.toString().indexOf(toString);
      int index = content.toString().indexOf(fromString);
      if (toIndex == -1 && index > 0)
       {
        content.delete(index, index + fromString.length());
        content.insert(index, toString);
        writeFileContent(fileName, content);
       }
     }
    catch (Exception e)
     {
      System.out.println(e);
      e.printStackTrace();
     }
   }

  /**
   * Insert a string in a specified file.
   *
   * @param fileName The name of the file that is to be updated.
   * @param insertString The text to be inserted.
   * @param insertKeyWord The keyword string. If the keyword exist in the file, no need to insert the insertString.
   * @param markString The last occurance of string in the file indicate where to insert.
   * @param position Either "before" or "after" indicating insert the new string before or after the mark string.
   */
  public static void insertIntoFile(String fileName, String insertString, String insertKeyWord, String markString, String position)
   {
    try
     {
      StringBuffer content = getFileContent(fileName);
      int toIndex = content.toString().indexOf(insertKeyWord);
      int index = content.toString().lastIndexOf(markString);
      if (toIndex == -1 && index > 0 && (position.equalsIgnoreCase("after") || position.equalsIgnoreCase("before")) )
       {
        if ( position.equalsIgnoreCase("after") ) {
            index = index + markString.length();
        }
        content.insert(index, insertString);
        writeFileContent(fileName, content);
       }
     }
    catch (Exception e)
     {
      System.out.println(e);
      e.printStackTrace();
     }
   }

  /**
   * Add an EJB module to application.xml. application.xml is located in the META-INF directory of the
   * ear directory.
   *
   * @param xmlFile Content of application.xml.
   * @param jarFile Name of EJB jar file.
   */
  public static void addEJBModuleToApplicationXML(StringBuffer xmlFile, EJBModuleDefinition ejbModuleDefinition)
   {
    String jarFile = ejbModuleDefinition._jarFile;
    // If the jar is already in application.xml, remove it.
    removeEJBModuleFromApplicationXML(xmlFile, jarFile);
    // locate the first module.
    int index = xmlFile.toString().indexOf("<module ");
    if (index == -1)
     {
      System.out.println("There is a problem with application.xml. No modules were found.");
      System.exit(-1);
     }
    // create a unique id
    long newTime = new Date().getTime();
    while (_lastTime >= newTime)
     {
      newTime++;
     }
    _lastTime = newTime;
    // Insert the new module into the file.
    String insertString = "<module id=\"EjbModule_" + newTime + "\">" + _newLine +
                    "         <ejb>" + jarFile + "</ejb>" + _newLine +
                    "      </module>" + _newLine +
                    "      ";
    xmlFile.insert(index, insertString);
   }

  /**
   * Remove an EJB module from application.xml.
   *
   * @param xmlFile Content of application.xml.
   * @param jarFile Name of EJB jar file.
   */
  public static void removeEJBModuleFromApplicationXML(StringBuffer xmlFile, String jarFile)
   {
    // loop in case the module exists multiple times
    while (true)
     {
      // locate the jar file.
      int jarFileIndex = xmlFile.toString().indexOf(">" + jarFile);
      // remove the module element.
      if (!removeElementFromXML(xmlFile, "module", jarFileIndex))
       {
        break;
       }
     }
   }

  /**
   * Add an EJB module to xml config file.
   *
   * @param fileContent XML file content to be updated.
   * @param ejbModuleDefinition The definition of the ejbModule.
   */
  public static void addEJBModuleToConfig(StringBuffer xmlFile, EJBModuleDefinition ejbModuleDefinition)
   {
    // Remove old module definition.
    removeEJBModuleFromConfig(xmlFile, ejbModuleDefinition._jarFile);
    // locate the first ejb module
    int ejbModuleIndex = xmlFile.toString().indexOf("<ejb-module ");
    if (ejbModuleIndex == -1)
     {
      System.out.println("There is a problem with the configuration file. No ejb modules were found.");
      System.exit(-1);
     }
    // extract the first ejb module into a string buffer
    ejbModuleIndex = xmlFile.toString().lastIndexOf("\n", ejbModuleIndex);
    if (xmlFile.charAt(ejbModuleIndex - 1) == '\r')
     {
      ejbModuleIndex--;
     }
    int ejbModuleEndIndex = xmlFile.toString().indexOf("</ejb-module>", ejbModuleIndex) + "</ejb-module>".length();
    StringBuffer newEJBModule = new StringBuffer(xmlFile.substring(ejbModuleIndex, ejbModuleEndIndex));
    // replace the ejb module name with the new name
    replaceAttribute(newEJBModule, "ejb-module", "name", ejbModuleDefinition._ejbModuleName);
    // replace the jar-file element content with the new jar file name.
    replaceElementContent(newEJBModule, "jar-file", ejbModuleDefinition._jarFile);
    // Remove all of old EJBs
    while (true)
     {
      int index = newEJBModule.toString().indexOf("enterprise-bean-binding");
      if (!removeElementFromXML(newEJBModule, "enterprise-bean-binding", index))
       {
        break;
       }
     }
    // add the new ejbs.
    int insertIndex = newEJBModule.toString().indexOf("</data-source>") + "</data-source>".length();
    for (int i = 0; i < ejbModuleDefinition._ejbs.size(); i++)
     {
      EJBDefinition ejbDefinition = (EJBDefinition) ejbModuleDefinition._ejbs.elementAt(i);
      String enterpriseBeanName = ejbDefinition._enterpriseBean;
      String jndiName = ejbDefinition._jndiName;
      String insertString = "\n" +
                            "                <enterprise-bean-binding name=\"" + ejbDefinition._bindingName + "\">\n" +
                            "                    <enterprise-bean-name>" + enterpriseBeanName + "</enterprise-bean-name>\n" +
                            "                    <jndi-name>" + jndiName + "</jndi-name>\n" +
                            "                </enterprise-bean-binding>";
      newEJBModule.insert(insertIndex, insertString);
     }
    // insert the ejb module into the xml file.
    xmlFile.insert(ejbModuleIndex, newEJBModule);
   }

  /**
   * Remove EJB module from xml config file.
   *
   * @param fileContent XML file content to be updated.
   * @param jarFile The jar file name.
   */
  public static void removeEJBModuleFromConfig(StringBuffer xmlFile, String jarFile)
   {
    // loop in case the module exists multiple times
    while (true)
     {
      // locate the jar file.
      int jarFileIndex = xmlFile.toString().indexOf(">" + jarFile);
      // remove the ejb-module element from the XML
      if (!removeElementFromXML(xmlFile, "ejb-module", jarFileIndex))
       {
        break;
       }
     }
   }

  /**
   * Remove element from XML file.
   *
   * @param xmlFile XML file content to be updated.
   * @param elementName Name of element.
   * @param index Position with the element to be deleted.
   *
   * @return true if the element was removed.
   */
  public static boolean removeElementFromXML(StringBuffer xmlFile, String elementName, int index)
   {
    if (index == -1)
     {
      // if the start index is -1, return false
      return false;
     }
    // look for the start of the element.
    index = xmlFile.toString().lastIndexOf("<" + elementName + " ", index);
    if (index == -1)
     {
      // if the element is not found, return false
      return false;
     }
    // go to the beginning of the current line
    index = xmlFile.toString().lastIndexOf("\n", index);
    if (xmlFile.charAt(index - 1) == '\r')
     {
      index--;
     }
    // Find the end of the element
    String endElement = "</" + elementName + ">";
    int endIndex = xmlFile.toString().indexOf(endElement, index) + endElement.length();
    // Delete the element
    xmlFile.delete(index, endIndex);
    // Return true to indicate the element was deleted
    return true;
   }

  /**
   * Get an attribute value from an XML document.
   *
   * @param content XML file content.
   * @param attribute The name of the attribute.
   *
   * @return The value of the attribute.
   */
  public static String getAttribute(StringBuffer content, String attribute)
   {
    return getAttribute(content, attribute, null);
   }

  /**
   * Get an attribute value from an XML document. This method allow you to specify a string that
   * marks the point where you want to start looking for the attribute.
   *
   * @param content XML file content.
   * @param attribute The name of the attribute.
   * @param startAtSTring A string to look for as the starting point for the search.
   *
   * @return The value of the attribute.
   */
  public static String getAttribute(StringBuffer content, String attribute, String startAtString)
   {
    int startChar = 0;
    if (startAtString != null)
     {
      startChar = content.toString().indexOf(startAtString);
      if (startChar == -1)
       {
        startChar = 0;
       }
     }
    int index = content.toString().indexOf(attribute, startChar);
    if (index != -1)
     {
      index += attribute.length() + 2;
      for (int i = index; i < content.length(); i++)
       {
        if (content.charAt(i) == '"')
         {
          return content.toString().substring(index, i);
         }
       }
     }
    return null;
   }

  /**
   * Replace an attribute's value.
   *
   * @param content XML file content.
   * @param element The name of the element.
   * @param attribute The name of the attribute.
   * @param newValue The name value for the attribute.
   */
  public static void replaceAttribute(StringBuffer content, String element, String attribute, String newValue)
   {
    int startChar = content.toString().indexOf("<" + element);
    if (startChar == -1)
     {
      System.out.println("Element " + element + " not found.");
      System.exit(-1);
     }
    int index = content.toString().indexOf(attribute, startChar);
    if (index != -1)
     {
      index += attribute.length() + 2;
      int endIndex = content.toString().indexOf('"', index);
      content.delete(index, endIndex);
      content.insert(index, newValue);
     }
   }

  /**
   * Replace the content of an element.
   *
   * @param content XML file content.
   * @param element The name of the attribute.
   * @param newContent The new content for the element.
   */
  public static void replaceElementContent(StringBuffer content, String element, String newContent)
   {
    int startChar = content.toString().indexOf("<" + element);
    if (startChar == -1)
     {
      System.out.println("Element " + element + " not found.");
      System.exit(-1);
     }
    int index = content.toString().indexOf(">", startChar) + 1;
    int endIndex = content.toString().indexOf('<', index);
    content.delete(index, endIndex);
    content.insert(index, newContent);
   }

  /**
   * Export configuration from WAS.
   *
   * @param instanceInfo Information about the instance.
   *
   * @return The content of the configuration file.
   */
  public static StringBuffer exportXMLConfig(InstanceInfo instanceInfo)
   {
    System.out.println("Export configuration");
    String enterpriseApplication = "WebSphere Commerce Enterprise Application - " + instanceInfo._name;
    if (isOS400())
     {
      // try to stop the commerce application server
      enterpriseApplication = instanceInfo._name + " - WebSphere Commerce Enterprise Application";
     }
    // Call XMLConfig to export the configuration
    String exportFileName = instanceInfo._workingDirectory + _fileSeparator + "xmlConfigExport.xml";
    File exportFile = new File(exportFileName);
    exportFile.delete();
    String command = _xmlconfig;
    command += " -export " + exportFileName +
               " -adminNodeName " + instanceInfo._nodeName +
               " -substitute \"Enterprise_Application_Name=" + enterpriseApplication + "\"" +
               " -nameServiceHost " + _nameServiceHost +
               " -nameServicePort " + instanceInfo._wasPort +
               " -traceString com.ibm.websphere.XMLConfig.*=all=enabled" +
               " -traceFile " + instanceInfo._workingDirectory + _fileSeparator + "xmlConfigExport.log" +
               " -partial " + _wcs + _fileSeparator + "xml" + _fileSeparator + "config" + _fileSeparator + "was.export.app.xml";
    executeCommand(command, true, instanceInfo._workingDirectory);
    if (!exportFile.exists())
     {
      // if the export file does not exist then exit
      System.out.println("Configuration export failed.");
      System.exit(-1);
     }
    return getFileContent(exportFileName);
   }

  /**
   * Delete enterprise application configuration from WAS.
   *
   * @param instanceInfo Information about the instance.
   */
  public static void deleteXMLConfig(InstanceInfo instanceInfo)
   {
    System.out.println("Delete configuration");
    // Rename the ear directory to get it out of the way. This will exit the install if it can not be renamed.
    rename(instanceInfo._ear, instanceInfo._ear + ".bak", false);
    // Call XMLConfig to delete the configuration.
    String command = _xmlconfig +
                     " -import " + _wcs + _fileSeparator + "xml" + _fileSeparator + "config" + _fileSeparator + "was.delete.app.xml" +
                     " -adminNodeName " + instanceInfo._nodeName +
                     " -substitute DRIVER_INSTANCE_NAME=" + instanceInfo._name +
                     " -nameServiceHost " + _nameServiceHost +
                     " -nameServicePort " + instanceInfo._wasPort +
                     " -traceString com.ibm.websphere.XMLConfig.*=all=enabled" +
                     " -traceFile " + instanceInfo._workingDirectory + _fileSeparator + "xmlConfigDelete.log";
    executeCommand(command, true, instanceInfo._workingDirectory);
    // Restore the ear directory back to its original name
    rename(instanceInfo._ear + ".bak", instanceInfo._ear, false);
   }

  /**
   * Import enterprise application configuration into WAS.
   *
   * @param instanceInfo Information about the instance.
   * @param fileContent Content of the XML config file.
   */
  public static void importXMLConfig(InstanceInfo instanceInfo, StringBuffer fileContent)
   {
    System.out.println("Import configuration");
    // Write out the content of the configuartion file.
    String importFileName = instanceInfo._workingDirectory + _fileSeparator + "xmlConfigImport.xml";
    writeFileContent(importFileName, fileContent);
    // Call XMLConfig to import the application.
    String command = _xmlconfig +
                     " -import " + importFileName +
                     " -adminNodeName " + instanceInfo._nodeName +
                     " -nameServiceHost " + _nameServiceHost +
                     " -nameServicePort " + instanceInfo._wasPort +
                     " -traceString com.ibm.websphere.XMLConfig.*=all=enabled" +
                     " -traceFile " + instanceInfo._workingDirectory + _fileSeparator + "xmlConfigImport.log";
    executeCommand(command, true, instanceInfo._workingDirectory);
   }

  /**
   * Patch the web.xml files for the stores and tools web modules to work around problem with
   * servlets being initialized before the EJB modules are started.
   */
  public static void patchWebModules(InstanceInfo instanceInfo)
   {
    System.out.println("Patching web modules.");
    updateFile(instanceInfo._ear + "/wcstores.war/WEB-INF/web.xml",
                             "<load-on-startup>1</load-on-startup>",
                             "<load-on-startup></load-on-startup>");
    updateFile(instanceInfo._ear + "/wctools.war/WEB-INF/web.xml",
                             "<load-on-startup>1</load-on-startup>",
                             "<load-on-startup></load-on-startup>");
   }

  /**
   * Add a new wsad java project to the enterprise application
   */
  public static void addWSADJavaProject(InstanceInfo instanceInfo, String projectName, String URIName)
   {
    long t = System.currentTimeMillis();

    // modify <workspace>\WebSphereCommerceServer\META-INF\.modulemaps
    System.out.println("Updating Module Mapping: <workspace>\\WebSphereCommerceServer\\META-INF\\.modulemaps");
    insertIntoFile(instanceInfo._workspace + "/WebSphereCommerceServer/META-INF/.modulemaps",
                                 "  <utilityJARMappings xmi:id=\"UtilityJARMapping_" + t + "\" projectName=\"" + projectName + "\" uri=\"" + URIName + "\"/>" + _newLine,
                                 projectName,
                                 "</modulemap:EARProjectMap>",
                                 "before");

    // modify <workspace>\WebSphereCommerceServer\.project
    System.out.println("Updating project reference of WebSphereCommerceServer.");
    insertIntoFile(instanceInfo._workspace + "/WebSphereCommerceServer/.project",
                                 "   <project>" + projectName + "</project>" + _newLine + "        ",
                                 projectName,
                                 "</projects>",
                                 "before");

    // Updating Jar Dependency on war modules to include the new java project
    // - modify <workspace>\wcstores\Web Content\META-INF\MANIFEST.MF
    // - modify <workspace>\wctools\Web Content\META-INF\MANIFEST.MF
    System.out.println("Updating Jar Dependency on war modules.");
    updateFile(instanceInfo._workspace + "/wcstores/Web Content/META-INF/MANIFEST.MF",
                             "lib/ivjfix.jar",
                             "lib/ivjfix.jar " + URIName);
    updateFile(instanceInfo._workspace + "/wctools/Web Content/META-INF/MANIFEST.MF",
                             "lib/ivjfix.jar",
                             "lib/ivjfix.jar " + URIName);
   }

  /**
   * Add a new wsad ejb project to Enterprise Application Server
   */
  public static void addWSADEJBProject(InstanceInfo instanceInfo, String projectName, String URIName)
   {
    long t = System.currentTimeMillis();

    // modify <workspace>\WebSphereCommerceServer\META-INF\application.xml
    System.out.println("Updating Application Deployment Descriptor: <workspace>\\WebSphereCommerceServer\\META-INF\\application.xml");
    insertIntoFile(instanceInfo._workspace + "/WebSphereCommerceServer/META-INF/application.xml",
                                 _newLine + "      <module id=\"EjbModule_" + t +"\">" +
                                 _newLine + "        <ejb>" + URIName + "</ejb>" +
                                 _newLine + "      </module>",
                                 URIName,
                                 "</module>",
                                 "after");

    // modify <workspace>\WebSphereCommerceServer\META-INF\.modulemaps
    System.out.println("Updating Module Mapping: <workspace>\\WebSphereCommerceServer\\META-INF\\.modulemaps");
    insertIntoFile(instanceInfo._workspace + "/WebSphereCommerceServer/META-INF/.modulemaps",
                                 _newLine + "  <mappings xmi:id=\"ModuleMapping_" + t + "\" projectName=\"" + projectName + "\">"  +
                                 _newLine + "    <module xmi:type=\"application:EjbModule\" href=\"META-INF/application.xml#EjbModule_" + t + "\"/>"  +
                                 _newLine + "  </mappings>",
                                 projectName,
                                 "</mappings>",
                                 "after");

    // modify <workspace>\wcs.wte\WebSphereCommerceServer.wsc\server-cfg.xml
    System.out.println("Updating server configuration file: <workspace>\\wcs.wte\\WebSphereCommerceServer.wsc\\server-cfg.xml");
    insertIntoFile(instanceInfo._workspace + "/wcs.wte/WebSphereCommerceServer.wsc/server-cfg.xml",
                                 " EJBModuleRef_" + t,
                                 URIName,
                                 "\" defaultDatasource",
                                 "before");
    insertIntoFile(instanceInfo._workspace + "/wcs.wte/WebSphereCommerceServer.wsc/server-cfg.xml",
                                 "  <modules xmi:type=\"applicationserver:EJBModuleRef\" xmi:id=\"EJBModuleRef_" + t + "\" uri=\"" + URIName + "\"/>" + _newLine +
                                 "    ",
                                 URIName,
                                 "</installedApps>",
                                 "before");

    // modify <workspace>\wcs.wte\WebSphereCommerceServer.wsc\memento.xml
    System.out.println("Updating server configuration file: <workspace>\\wcs.wte\\WebSphereCommerceServer.wsc\\memento.xml");
    insertIntoFile(instanceInfo._workspace + "/wcs.wte/WebSphereCommerceServer.wsc/memento.xml",
                                 "   <memento-map-entry key=\"" + URIName + "\" value=\"com.ibm.etools.server.j2ee.ejb11:" + projectName + "\"/>" + _newLine +
                                 "   ",
                                 URIName,
                                 "</memento-map>",
                                 "before");


    // modify <workspace>\WebSphereCommerceServer\.project
    System.out.println("Updating project reference of WebSphereCommerceServer.");
    insertIntoFile(instanceInfo._workspace + "/WebSphereCommerceServer/.project",
                                 "   <project>" + projectName + "</project>" + _newLine +
                                 "        ",
                                 projectName,
                                 "</projects>",
                                 "before");

   }

  /**
   * Import new projects into WSAD workspace
   */
  public static void importProjects(InstanceInfo instanceInfo)
   {
    System.out.println("Import new projects into WSAD workspace.");
    String command = _wcs;
    if (isWindows())
     {
      command += "\\bin\\profileSampleImportWorkSpace.bat ";
     }
    command += instanceInfo._workspace + " " + _wcs + "\\bin\\importProject.xml";
    executeCommand(command, true, instanceInfo._workingDirectory);
   }

  /**
   * Add Vector of EJBs to the application.
   *
   * @param instanceInfo Information about the instance.
   * @param ejbs Vector of EJBModuleDefinition objects. The jars must be in the ear directory.
   */
  public static void addEJBsToEAR(InstanceInfo instanceInfo, Vector ejbs)
   {
    // Export the current configuration for the Commerce EAR
    StringBuffer configXMLContent = exportXMLConfig(instanceInfo);

    // Ensure that ear file name points to the installed ear directory
    replaceElementContent(configXMLContent, "ear-file-name", instanceInfo._ear);

    // Update the configuration xml and application.xml with the new EJB module
    StringBuffer applicationXMLContent = getFileContent(instanceInfo._ear + "/META-INF/application.xml");
    for (int i = 0; i < ejbs.size(); i++)
     {
      EJBModuleDefinition ejbModuleDefinition = (EJBModuleDefinition) ejbs.elementAt(i);
      addEJBModuleToConfig(configXMLContent, ejbModuleDefinition);
      addEJBModuleToApplicationXML(applicationXMLContent, ejbModuleDefinition);
     }
    writeFileContent(instanceInfo._ear + "/META-INF/application.xml", applicationXMLContent);

    // Delete the old configuration
    deleteXMLConfig(instanceInfo);

    // Import the new configuration
    importXMLConfig(instanceInfo, configXMLContent);
   }

  /**
   * Information about a Commerce instance.
   */
  final public static class InstanceInfo
   {
    /**
     * Name of the instance.
     */
    public String _name;

    /**
     * Path to <instance>.xml file.
     */
    public String _configXML;

    /**
     * Name of the database.
     */
    public String _databaseName;

    /**
     * Database user id.
     */
    public String _databaseUserId;

    /**
     * Database password.
     */
    public String _databasePassword;

    /**
     * Database type.
     */
    public String _databaseType;

    /**
     * Path to commerce ear directory.
     */
    public String _ear;

    /**
     * Path to massloader log configuration file.
     */
    public String _wcaLogger;

    /**
     * Path to log directory.
     */
    public String _errorLogDir;

    /**
     * Node name used by this instance.
     */
    public String _nodeName;

    /**
     * Port used to connect to WAS.
     */
    public String _wasPort;

    /**
     * OS400 instance name.
     */
    public String _os400WASInstanceName;

    /**
     * Workspace directory.
     */
    public String _workspace;

    /**
     * Working directory.
     */
    public String _workingDirectory;
   }

  /**
   * Definition of an EJB Module.
   */
  final public static class EJBModuleDefinition
   {
    /**
     * Name of EJB module.
     */
    public String _ejbModuleName;

    /**
     * Name of EJB jar file.
     */
    public String _jarFile;

    /**
     * Vector of EJBDefinition objects.
     */
    public Vector _ejbs = new Vector();
   }

  /**
   * Definition of an EJB.
   */
  final public static class EJBDefinition
   {
    /**
     * Name of enterprise bean.
     */
    public String _enterpriseBean;

    /**
     * Binding name.
     */
    public String _bindingName;

    /**
     * JNDI name for enterprise bean.
     */
    public String _jndiName;
   }
 }
