//*-------------------------------------------------------------------
//* Licensed Materials - Property of IBM
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2002
//*
//* US Government Users Restricted Rights - Use, duplication or
//* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//*
//*-------------------------------------------------------------------
//*
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its reliability,
//* serviceability or functionality.
//*
//*-------------------------------------------------------------------
//*
//* This class is will contains helper methods that deal with XML documents.
//*
//*-------------------------------------------------------------------

package com.mycompany.install.util;

import com.ibm.xml.parsers.NonValidatingDOMParser;
import com.mycompany.install.InstallHelper;
import java.io.InputStream;
import java.util.Hashtable;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * This class contains a number of utility methods that are used during the install.
 * Because this class is move to the utilities jar and loaded dynamically, it can contain
 * dependencies on the xml4j.jar.
 */
public final class InstallEJBHelper
 {
  /**
   * Parse an XML document.
   *
   * @param inputStream Document input stream.
   *
   * @returns A document object or null if the document could not be parsed.
   */
  public static Document parseDocument(InputStream inputStream)
   {
    NonValidatingDOMParser parser = new NonValidatingDOMParser();
    try
     {
      parser.parse(new InputSource(inputStream));
     }
    catch (SAXException e)
     {
      System.out.println(e.getMessage());
      return null;
     }
    catch (Exception e)
     {
      e.printStackTrace();
      return null;
     }
    if (parser != null)
     {
      return parser.getDocument();
     }
    return null;
   }

  /**
   * Parse a document that is contained in a zip file. This method will exit if it fails to parse the document.
   *
   * @param zipFileName Name of the zip file.
   * @param zipEnryName Path to the document contained in the zip file.
   *
   * @returns A Document object if successful.
   */
  public static Document parseDocument(String zipFileName, String zipEntryName)
   {
    ZipFile zipFile = null;
    try
     {
      zipFile = new ZipFile(zipFileName.trim());
     }
    catch (Exception e)
     {
      System.out.println(zipFileName + " could not be opened.");
      System.exit(-1);
     }
    try
     {
      ZipEntry zipEntry = zipFile.getEntry(zipEntryName);
      InputStream inputStream = zipFile.getInputStream(zipEntry);
      Document document = parseDocument(inputStream);
      inputStream.close();
      zipFile.close();
      if (document == null)
       {
        System.out.println("Problem parsing " + zipEntryName + " in " + zipFileName + ".");
        System.exit(-1);
       }
      return document;
     }
    catch (Exception e)
     {
      e.printStackTrace();
      System.out.println("Problem parsing " + zipEntryName + " in " + zipFileName + ".");
      System.exit(-1);
     }
    return null;
   }

  /**
   * Get the value of a child element.
   *
   * @param element The parent element.
   * @param tagName The name of the child element.
   *
   * @returns The value of the child element.
   */
  public static String getChildValue(Element element, String tagName)
   {
    NodeList nodeList = element.getElementsByTagName(tagName);
    if (nodeList != null && nodeList.getLength() > 0)
     {
      return nodeList.item(0).getFirstChild().getNodeValue().trim();
     }
    return null;
   }

  /**
   * Construct an EJBDefinition object from an ejb element. The ejb element is either a session or entity element
   * that was pulled from ejb-jar.xml. This method will exit if the JNDI name can not be determined.
   *
   * @param bindingNameMappings Hashtable of binding name mappings.
   * @param jndiMappings Hashtable of jndi mappings.
   * @param element Element object of ejb.
   *
   * @returns An EJBDefinition object.
   */
  public static InstallHelper.EJBDefinition constructEJBDefinition(Hashtable bindingNameMappings, Hashtable jndiMappings, Element element)
   {
    // Construct EJB Definition object
    InstallHelper.EJBDefinition ejbDefinition = new InstallHelper.EJBDefinition();
    // Set the enterprise bean name
    ejbDefinition._enterpriseBean = getChildValue(element, "ejb-name");
    // Get the id of the enterprise bean
    String id = element.getAttribute("id");
    // Use the id to determine the binding name.
    ejbDefinition._bindingName = (String) bindingNameMappings.get(id);
    if (ejbDefinition._bindingName == null || ejbDefinition._bindingName.length() == 0)
     {
      // Exit if the binding name could not be determined.
      System.out.println("binding name could not be determined for "+id);
      System.exit(-1);
     }
    // Use the id to determine the jndi name.
    ejbDefinition._jndiName = (String) jndiMappings.get(id);
    if (ejbDefinition._jndiName == null || ejbDefinition._jndiName.length() == 0)
     {
      // Exit if the jndi name could not be determined.
      System.out.println("jndi name could not be determined for "+id);
      System.exit(-1);
     }
    return ejbDefinition;
   }

  /**
   * Construct an EJBModuleDefinition object for an ejb jar. The ejb jar must be in the Commerce ear directory.
   *
   * @param instanceInfo Information about the Commerce instance.
   * @param ejbJar The name of the ejb jar.
   *
   * @returns An EJBModuleDefinition object.
   */
  public static InstallHelper.EJBModuleDefinition getEJBModuleDefinition(InstallHelper.InstanceInfo instanceInfo, String ejbJar)
   {
    // Parse ibm-ejb-jar-bnd.xmi to determine the jndi names that will be used.
    Document document = parseDocument(instanceInfo._ear + "/" + ejbJar, "META-INF/ibm-ejb-jar-bnd.xmi");
    NodeList elements = document.getElementsByTagName("ejbBindings");
    Hashtable jndiMappings = new Hashtable();
    Hashtable bindingNameMappings = new Hashtable();
    if (elements != null)
     {
      for (int i = 0; i < elements.getLength(); i++)
       {
        Element element = (Element) elements.item(i);
        String bindingName = element.getAttribute("xmi:id");
        String jndiName = instanceInfo._name + element.getAttribute("jndiName");
        NodeList nodeList = element.getElementsByTagName("enterpriseBean");
        if (nodeList != null && nodeList.getLength() > 0)
         {
          Element enterpriseBeanElement = (Element) nodeList.item(0);
          String href = enterpriseBeanElement.getAttribute("href");
          String ejbId = href.substring(href.lastIndexOf("#") + 1);
          bindingNameMappings.put(ejbId, bindingName);
          jndiMappings.put(ejbId, jndiName);
         }
       }
     }

    // Parse ejb-jar.xml to find all of the EJBs.
    document = parseDocument(instanceInfo._ear + "/" + ejbJar, "META-INF/ejb-jar.xml");
    InstallHelper.EJBModuleDefinition ejbModuleDefinition = new InstallHelper.EJBModuleDefinition();
    ejbModuleDefinition._ejbModuleName = getChildValue(document.getDocumentElement(), "display-name");
    if (ejbModuleDefinition._ejbModuleName == null)
     {
      ejbModuleDefinition._ejbModuleName = ejbJar;
     }
    ejbModuleDefinition._jarFile = ejbJar;
    // get all of the entity beans
    elements = document.getElementsByTagName("entity");
    if (elements != null)
     {
      for (int i = 0; i < elements.getLength(); i++)
       {
        ejbModuleDefinition._ejbs.addElement(constructEJBDefinition(bindingNameMappings, jndiMappings, (Element) elements.item(i)));
       }
     }
    // get all of the session beans
    elements = document.getElementsByTagName("session");
    if (elements != null)
     {
      for (int i = 0; i < elements.getLength(); i++)
       {
        ejbModuleDefinition._ejbs.addElement(constructEJBDefinition(bindingNameMappings, jndiMappings, (Element) elements.item(i)));
       }
     }
    return ejbModuleDefinition;
   }

  /**
   * Main entry point for this class. This method expects a list of EJB jars that have already been placed in the
   * commerce ear directory. On Windows, you can also specify -wcs <commerce root> as a parameter.
   *
   * @param args List of ejb jars.
   */
  public static void main(String[] args)
   {
    if (args == null || args.length == 0)
     {
      System.out.println("Specify a list of EJB jars that you want to add to the IBM WebSphere Commerce application.");
      System.out.println("The EJB modules must be placed in the ear directory.");
      System.exit(0);
     }

    // On windows, check for -wcs parameter
    if (InstallHelper.isWindows())
     {
      for (int i = 0; i < args.length; i++)
       {
        if ("-wcs".equals(args[i]))
         {
          i++;
          if (i < args.length)
           {
            // Initialize the root directory to be used.
            InstallHelper.initWCS(args[i]);
           }
         }
       }
     }
    // Initialize the install helper.
    InstallHelper.init(InstallHelper.INSTALL_DESTINATION_WAS, true);
    // Loop through the instances and deploy the list of ejbs.
    for (int i = 0; i < InstallHelper._instances.size(); i++)
     {
      InstallHelper.InstanceInfo instanceInfo = (InstallHelper.InstanceInfo) InstallHelper._instances.elementAt(i);
      Vector ejbDefinitions = new Vector();
      if (args != null && args.length > 0)
       {
        for (int j = 0; j < args.length; j++)
         {
          if (args[j].startsWith("-"))
           {
            j++;
           }
          else
           {
            ejbDefinitions.addElement(getEJBModuleDefinition(instanceInfo, args[j]));
           }
         }
       }
      InstallHelper.addEJBsToEAR(instanceInfo, ejbDefinitions);
     }
    System.exit(0);
   }
 }
